@@ -3,7 +3,7 @@
  *  Copyright (c) 2002, 2007 Sam Hocevar <sam@zoy.org>
  *                All Rights Reserved
  *
- *  $Id: opts.h 255 2007-01-27 19:05:46Z sam $
+ *  $Id: opts.h 301 2007-07-10 15:23:18Z sam $
  *
  *  This program is free software. It comes without any warranty, to
  *  the extent permitted by applicable law. You can redistribute it
@@ -20,13 +20,14 @@ struct opts
 {
     char **oldargv;
     char **newargv;
-    char *fuzzing, *bytes, *protect, *refuse;
+    char *fuzzing, *bytes, *pick, *protect, *refuse;
     uint32_t seed;
     uint32_t endseed;
     double minratio;
     double maxratio;
     int quiet;
     int maxbytes;
+    int maxcpu;
     int md5;
     int checkexit;
     int verbose;
