@@ -3,7 +3,7 @@
  *  Copyright (c) 2002, 2007 Sam Hocevar <sam@zoy.org>
  *                All Rights Reserved
  *
- *  $Id: zzuf.c 287 2007-03-01 23:52:00Z sam $
+ *  $Id: zzuf.c 302 2007-07-10 15:31:40Z sam $
  *
  *  This program is free software. It comes without any warranty, to
  *  the extent permitted by applicable law. You can redistribute it
@@ -76,13 +76,19 @@
 #endif
 
 #if defined RLIMIT_AS
-#   define ZZUF_RLIMIT_CONST RLIMIT_AS
+#   define ZZUF_RLIMIT_MEM RLIMIT_AS
 #elif defined RLIMIT_VMEM
-#   define ZZUF_RLIMIT_CONST RLIMIT_VMEM
+#   define ZZUF_RLIMIT_MEM RLIMIT_VMEM
 #elif defined RLIMIT_DATA
-#   define ZZUF_RLIMIT_CONST RLIMIT_DATA
+#   define ZZUF_RLIMIT_MEM RLIMIT_DATA
 #else
-#   undef HAVE_SETRLIMIT
+#   undef ZZUF_RLIMIT_MEM
+#endif
+
+#if defined RLIMIT_CPU
+#   define ZZUF_RLIMIT_CPU RLIMIT_CPU
+#else
+#   undef ZZUF_RLIMIT_CPU
 #endif
 
 /* We use file descriptor 17 as the debug channel */
@@ -151,13 +157,18 @@ int main(int argc, char *argv[])
 #else
 #   define OPTSTR_REGEX ""
 #endif
-#if defined HAVE_SETRLIMIT
-#   define OPTSTR_RLIMIT "M:"
+#if defined HAVE_SETRLIMIT && defined ZZUF_RLIMIT_MEM
+#   define OPTSTR_RLIMIT_MEM "M:"
 #else
-#   define OPTSTR_RLIMIT ""
+#   define OPTSTR_RLIMIT_MEM ""
 #endif
-#define OPTSTR OPTSTR_REGEX OPTSTR_RLIMIT \
-            "Ab:B:C:dD:f:F:imnP:qr:R:s:ST:vxhV"
+#if defined HAVE_SETRLIMIT && defined ZZUF_RLIMIT_CPU
+#   define OPTSTR_RLIMIT_CPU "T:"
+#else
+#   define OPTSTR_RLIMIT_CPU ""
+#endif
+#define OPTSTR OPTSTR_REGEX OPTSTR_RLIMIT_MEM OPTSTR_RLIMIT_CPU \
+                "Ab:B:C:dD:f:F:imnp:P:qr:R:s:St:vxhV"
 #define MOREINFO "Try `%s --help' for more information.\n"
         int option_index = 0;
         static struct myoption long_options[] =
@@ -184,13 +195,15 @@ int main(int argc, char *argv[])
             { "md5",         0, NULL, 'm' },
             { "max-memory",  1, NULL, 'M' },
             { "network",     0, NULL, 'n' },
+            { "pick",        1, NULL, 'p' },
             { "protect",     1, NULL, 'P' },
             { "quiet",       0, NULL, 'q' },
             { "ratio",       1, NULL, 'r' },
             { "refuse",      1, NULL, 'R' },
             { "seed",        1, NULL, 's' },
             { "signal",      0, NULL, 'S' },
-            { "max-time",    1, NULL, 'T' },
+            { "max-time",    1, NULL, 't' },
+            { "max-cpu",     1, NULL, 'T' },
             { "verbose",     0, NULL, 'v' },
             { "check-exit",  0, NULL, 'x' },
             { "help",        0, NULL, 'h' },
@@ -265,7 +278,7 @@ int main(int argc, char *argv[])
         case 'm': /* --md5 */
             opts->md5 = 1;
             break;
-#if defined HAVE_SETRLIMIT
+#if defined HAVE_SETRLIMIT && defined ZZUF_RLIMIT_MEM
         case 'M': /* --max-memory */
             setenv("ZZUF_MEMORY", "1", 1);
             opts->maxmem = atoi(myoptarg);
@@ -274,6 +287,9 @@ int main(int argc, char *argv[])
         case 'n': /* --network */
             setenv("ZZUF_NETWORK", "1", 1);
             break;
+        case 'p': /* --pick */
+            opts->pick = myoptarg;
+            break;
         case 'P': /* --protect */
             opts->protect = myoptarg;
             break;
@@ -296,9 +312,14 @@ int main(int argc, char *argv[])
         case 'S': /* --signal */
             setenv("ZZUF_SIGNAL", "1", 1);
             break;
-        case 'T': /* --max-time */
+        case 't': /* --max-time */
             opts->maxtime = (int64_t)(atof(myoptarg) * 1000000.0);
             break;
+#if defined HAVE_SETRLIMIT && defined ZZUF_RLIMIT_CPU
+        case 'T': /* --max-cpu */
+            opts->maxcpu = (int)(atof(myoptarg) + 0.5);
+            break;
+#endif
         case 'x': /* --check-exit */
             opts->checkexit = 1;
             break;
@@ -375,6 +396,8 @@ int main(int argc, char *argv[])
         setenv("ZZUF_FUZZING", opts->fuzzing, 1);
     if(opts->bytes)
         setenv("ZZUF_BYTES", opts->bytes, 1);
+    if(opts->pick)
+        setenv("ZZUF_PICK", opts->pick, 1);
     if(opts->protect)
         setenv("ZZUF_PROTECT", opts->protect, 1);
     if(opts->refuse)
@@ -427,6 +450,8 @@ static void loop_stdin(struct opts *opts)
         _zz_fuzzing(opts->fuzzing);
     if(opts->bytes)
         _zz_bytes(opts->bytes);
+    if(opts->pick)
+        _zz_pick(opts->pick);
     if(opts->protect)
         _zz_protect(opts->protect);
     if(opts->refuse)
@@ -691,11 +716,20 @@ static void clean_children(struct opts *opts)
                  && !(WTERMSIG(status) == SIGTERM
                        && opts->child[i].status == STATUS_SIGTERM))
         {
+            char const *message = "";
+
+            if(WTERMSIG(status) == SIGKILL && opts->maxmem >= 0)
+                message = " (memory exceeded?)";
+#   if defined SIGXCPU
+            else if(WTERMSIG(status) == SIGXCPU && opts->maxcpu >= 0)
+                message = " (CPU time exceeded?)";
+#   endif
+            else if(WTERMSIG(status) == SIGKILL && opts->maxcpu >= 0)
+                message = " (CPU time exceeded?)";
+
             finfo(stderr, opts, opts->child[i].seed);
             fprintf(stderr, "signal %i%s%s\n",
-                    WTERMSIG(status), sig2str(WTERMSIG(status)),
-                      (WTERMSIG(status) == SIGKILL && opts->maxmem >= 0) ?
-                      " (memory exceeded?)" : "");
+                    WTERMSIG(status), sig2str(WTERMSIG(status)), message);
             opts->crashes++;
         }
 #endif
@@ -882,13 +916,23 @@ static int run_process(struct opts *opts, int pipes[][2])
     }
 #endif
 
-#if defined HAVE_SETRLIMIT
+#if defined HAVE_SETRLIMIT && defined ZZUF_RLIMIT_MEM
     if(opts->maxmem >= 0)
     {
         struct rlimit rlim;
         rlim.rlim_cur = opts->maxmem * 1000000;
         rlim.rlim_max = opts->maxmem * 1000000;
-        setrlimit(ZZUF_RLIMIT_CONST, &rlim);
+        setrlimit(ZZUF_RLIMIT_MEM, &rlim);
+    }
+#endif
+
+#if defined HAVE_SETRLIMIT && defined ZZUF_RLIMIT_CPU
+    if(opts->maxcpu >= 0)
+    {
+        struct rlimit rlim;
+        rlim.rlim_cur = opts->maxcpu;
+        rlim.rlim_max = opts->maxcpu + 5;
+        setrlimit(ZZUF_RLIMIT_CPU, &rlim);
     }
 #endif
 
@@ -901,18 +945,34 @@ static int run_process(struct opts *opts, int pipes[][2])
     setenv("ZZUF_MAXRATIO", buf, 1);
 
 #if defined HAVE_FORK
-    libpath = malloc(len + strlen("/.libs/" FILENAME EXTRAINFO) + 1);
+    /* Meaningless but makes sure there is space for everything */
+    libpath = malloc(len + strlen(LIBDIR "/.libs/" FILENAME EXTRAINFO) + 1);
     strcpy(libpath, opts->oldargv[0]);
 
+    /* Replace "/path/binaryname" with "/path/.libs/libzzuf.$(EXT)"
+     *     and "binaryname" with ".libs/libzzuf.$(EXT)"
+     * Write the result in libpath. */
     tmp = strrchr(libpath, '/');
     strcpy(tmp ? tmp + 1 : libpath, ".libs/" FILENAME);
-    ret = access(libpath, R_OK);
 
-    strcpy(tmp ? tmp + 1 : libpath, ".libs/" FILENAME EXTRAINFO);
-    if(ret == 0)
-        setenv(PRELOAD, libpath, 1);
-    else
-        setenv(PRELOAD, LIBDIR "/" FILENAME EXTRAINFO, 1);
+    ret = access(libpath, R_OK);
+    if(ret < 0)
+        strcpy(libpath, LIBDIR "/" FILENAME);
+
+    /* OSF1 only */
+    strcat(libpath, EXTRAINFO);
+
+    /* Do not clobber previous LD_PRELOAD values */
+    tmp = getenv(PRELOAD);
+    if(tmp && *tmp)
+    {
+        char *bigbuf = malloc(strlen(tmp) + strlen(libpath) + 2);
+        sprintf(bigbuf, "%s:%s", tmp, libpath);
+        free(libpath);
+        libpath = bigbuf;
+    }
+
+    setenv(PRELOAD, libpath, 1);
     free(libpath);
 
     if(execvp(opts->newargv[0], opts->newargv))
@@ -1079,16 +1139,20 @@ static void usage(void)
     printf("Usage: zzuf [-AdimnqSvx] [-s seed|-s start:stop] [-r ratio|-r min:max]\n");
 #endif
     printf("              [-f fuzzing] [-D delay] [-F forks] [-C crashes] [-B bytes]\n");
-#if defined HAVE_SETRLIMIT
-    printf("              [-T seconds] [-M bytes] [-b ranges] [-P protect] [-R refuse]\n");
-#else
-    printf("              [-T seconds] [-b ranges] [-P protect] [-R refuse]\n");
+    printf("              [-t seconds] ");
+#if defined HAVE_SETRLIMIT && defined ZZUF_RLIMIT_CPU
+    printf(                           "[-T seconds] ");
 #endif
+#if defined HAVE_SETRLIMIT && defined ZZUF_RLIMIT_MEM
+    printf(                                        "[-M megabytes] ");
+#endif
+    printf(                                                       "[-b ranges]\n");
+    printf("              [-P protect] [-R refuse] [-p pick]");
 #if defined HAVE_REGEX_H
-    printf("              [-I include] [-E exclude] [PROGRAM [--] [ARGS]...]\n");
-#else
-    printf("              [PROGRAM [--] [ARGS]...]\n");
+    printf(                                                " [-I include] [-E exclude]");
 #endif
+    printf("\n");
+    printf("              [PROGRAM [--] [ARGS]...]\n");
     printf("       zzuf -h | --help\n");
     printf("       zzuf -V | --version\n");
     printf("Run PROGRAM with optional arguments ARGS and fuzz its input.\n");
@@ -1113,10 +1177,11 @@ static void usage(void)
     printf("  -I, --include <regex>     only fuzz files matching <regex>\n");
 #endif
     printf("  -m, --md5                 compute the output's MD5 hash\n");
-#if defined HAVE_SETRLIMIT
+#if defined HAVE_SETRLIMIT && defined ZZUF_RLIMIT_MEM
     printf("  -M, --max-memory <n>      maximum child virtual memory size in MB\n");
 #endif
     printf("  -n, --network             fuzz network input\n");
+    printf("  -p, --pick <list>         only fuzz Nth descriptor with N in <list>\n");
     printf("  -P, --protect <list>      protect bytes and characters in <list>\n");
     printf("  -q, --quiet               do not print children's messages\n");
     printf("  -r, --ratio <ratio>       bit fuzzing ratio (default %g)\n", DEFAULT_RATIO);
@@ -1125,7 +1190,8 @@ static void usage(void)
     printf("  -s, --seed <seed>         random seed (default %i)\n", DEFAULT_SEED);
     printf("      --seed <start:stop>   specify a seed range\n");
     printf("  -S, --signal              prevent children from diverting crashing signals\n");
-    printf("  -T, --max-time <n>        kill children that run for more than <n> seconds\n");
+    printf("  -t, --max-time <n>        kill children that run for more than <n> seconds\n");
+    printf("  -T, --max-cpu <n>         kill children that use more than <n> CPU seconds\n");
     printf("  -v, --verbose             print information during the run\n");
     printf("  -x, --check-exit          report processes that exit with a non-zero status\n");
     printf("  -h, --help                display this help and exit\n");
